/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */
#ifndef _LOG_H_
#define _LOG_H_

#include "common.h"

#if defined( _ADEONA_BUILD_RELEASE_ )
    #define LOG_INFO   1
    #define LOG_DEBUG  0
    #define LOG_CRYPTO 0
    #define LOG_WARNING 0  
    #define LOG_ERROR 0
    #define LOG_FATAL 1
    #define LOG_USETIMESTAMP 0
#else 
    #define LOG_INFO   1
    #define LOG_DEBUG  1
    #define LOG_CRYPTO 1
    #define LOG_WARNING 1  
    #define LOG_ERROR 1
    #define LOG_FATAL 1
    #define LOG_USETIMESTAMP 1
#endif

typedef enum
{
    ADEONA_INFO, 
    ADEONA_DEBUG, 
    ADEONA_CRYPTO,
    ADEONA_WARNING,  
    ADEONA_ERROR,
    ADEONA_FATAL     /* exit program */
} logdata_t;

typedef void ( *logfunc_t )( FILE* file, void* data );

int log_open( char* filename );
void log_close( void );

void log_msg( logdata_t type, char* fmtstr, ... );
void log_msg_bin( logdata_t type, uchar* data, int dlen, char* fmtstr, ... );
void log_msg_func( logdata_t type, logfunc_t interpreter, void* data );
void log_err( logdata_t type, const char* file, const char* function, const unsigned long line, char* fmtstr, ... );

void printout( char* fmtstr, ... );

#define  info( fmtstr, args... ) log_msg( ADEONA_INFO, fmtstr, ## args )
#define debug( fmtstr, args... ) log_msg( ADEONA_DEBUG, fmtstr, ## args )

#define  warn( fmtstr, args... ) log_err( ADEONA_WARNING, __FILE__, __FUNCTION__, __LINE__, fmtstr, ## args )
#define error( fmtstr, args... ) log_err( ADEONA_ERROR, __FILE__, __FUNCTION__, __LINE__, fmtstr, ## args )
#define fatal( fmtstr, args... ) log_err( ADEONA_FATAL, __FILE__, __FUNCTION__, __LINE__, fmtstr, ## args )

#endif // _LOG_H_

