/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */

#ifndef _CTXTCACHE_H_
#define _CTXTCACHE_H_

#include "common.h"
#include "crypto.h"
#include "options.h"
#include "location.h"

// Sentinel need not be too big since only worried about collisions 
// before next update
#define SENTINEL_SIZE   4   

// Amount "deep" we go with cacheseed looking for matching sentinel value.
// This should correspond to the maximum # of locations we expect to see between
// scheduled update times.
#define MAX_SENTINEL_TRIES 50

// PTXT_SIZE is size needed for the update header, plus whatever 
// is needed for location data
#define PTXT_SIZE	    ( sizeof( uint ) + LOC_DATA_UPPERBOUND )

// CTXT_SIZE just increases length of plaintexts by the MAC tag length
#define CTXT_SIZE	    ( PTXT_SIZE + HMAC_LEN )

// CTXT_QUEUE_LEN
#define CTXT_QUEUE_LEN  ( opts.ctxt_queue_size * sizeof( cache_elem_t ) )


// NOTE: not sure if this is best place for definition, but 
// should work for now.
typedef struct _update
{
    uint skipped; 
    location_t loc;
} update_t;

//typedef struct _fullhdr
//{
//    updatehdr_t up;
//    location_t  loc;
//} fullhdr_t;

typedef struct _cache_elem
{
    uchar sentinel[SENTINEL_SIZE];
    uchar ctxt[CTXT_SIZE];
} cache_elem_t;

//typedef struct _ctxt_cache 
//{ 
//  cache_elem* c;
//} ctxt_cache_t;

typedef struct _cache_keys
{
    prgkey_t   nextseed;
    uchar      sentinel[SENTINEL_SIZE]; 
    enckey_t   enckey;
    hmackey_t  hmackey;
} cache_keys_t;


cache_elem_t* ctxt_cache_alloc( void );
int ctxt_cache_fillrand( cache_elem_t* cache OUT );
int ctxt_cache_save( FILE* file IN, cache_elem_t* cache IN );
int ctxt_cache_load( FILE* file IN, cache_elem_t* cache OUT );
int ctxt_cache_del( cache_elem_t* cache IN );

int ctxt_cache_update( prgkey_t cacheseed INOUT, cache_elem_t* cache INOUT, update_t* update IN );
int ctxt_cache_getelem( prgkey_t cacheseed IN, cache_elem_t* cache IN, update_t* update OUT );

//int ctxt_cache_getsentinel( ctxt_cache_t* cache IN, uchar** sentinel OUT);


#endif // _CTXTCACHE_H_

