/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */
#include <openssl/hmac.h>

#include "hmac.h"
#include "log.h"

int hmac_compute( hmackey_t key IN, uchar* data IN, int dlen IN, uchar* mac OUT )
{
    if( sizeof( hmackey_t ) < 16 ) {
        warn( "using key that is less than 128-bits long" );
    }
    
    return hmacsha1_compute( key, sizeof( hmackey_t ), data, dlen, mac );
}

int hmac_verify( hmackey_t key IN, uchar* data IN, int dlen IN, uchar* mac IN )
{
    if( sizeof( hmackey_t ) < 16 ) {
        warn( "using key that is less than 128-bits long" );
    }

    return hmacsha1_verify( key, sizeof( hmackey_t ), data, dlen, mac );
}

int hmacsha1_compute( uchar* key IN, int klen IN, uchar* data IN, int dlen IN, uchar* mac OUT )
{
    if( key != NULL && data != NULL && klen > 0 && dlen > 0 && mac != NULL ) {
        
        unsigned int mlen = 0;
        
        HMAC( EVP_sha1(), key, klen, data, dlen, mac, &mlen );
    
        if( mlen == HMACSHA1_LEN ) {
            return SUCCESS;
        }
        else {
            memset( mac, 0x00, HMACSHA1_LEN );
            error( "could not generate hmac" );
            return FAILURE;
        }

    }
    else {
        error( "invalid input" );
        return FAILURE;
    }
}

int hmacsha1_verify( uchar* key IN, int klen IN, uchar* data IN, int dlen IN, uchar* mac IN )
{
    if( key != NULL && data != NULL && klen > 0 && dlen > 0 && mac != NULL ) {
        
        unsigned int mlen = 0;
        uchar computed[HMACSHA1_LEN];
        
        HMAC( EVP_sha1(), key, klen, data, dlen, computed, &mlen );
        
        if( mlen != HMACSHA1_LEN ) {
            memset( computed, 0x00, HMACSHA1_LEN );
            error( "could not generate hmac" );
            return FAILURE;
        }

        int i;
        for( i=0; i<HMACSHA1_LEN; i++ ) {
            if( computed[i] != mac[i] ) {
                warn( "invalid mac" );
                memset( computed, 0x00, HMACSHA1_LEN );
                return FAILURE;
            }
        }
        
        memset( computed, 0x00, HMACSHA1_LEN );
        return SUCCESS;
    }
    else {
        error( "invalid input" );
        return FAILURE;
    }

}


