/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */

#include "util.h"
#include "log.h"
#include "landmarks.h"

static struct in_addr* _landmarks = NULL;
static int _num_landmarks = 0;

/*
 * Load IP addresses of landmarks.
 * Expecting 1 IP address in xxx.xxx.xxx.xxx format per line
 *
 */ 
int landmarks_load( char* filename IN )
{
    if( filename != NULL ) {
        FILE* ipfile = fopen( filename, "r" );
        if( ipfile == NULL ) {
            error( "could not open landmarks file" );
            _landmarks = NULL;
            _num_landmarks = 0;
            return FAILURE;
        }
        
        assert( _landmarks == NULL && _num_landmarks == 0 );
        
        char ipstr[16];
        memset( ipstr, 0x00, sizeof( ipstr ) );
        while( fscanf( ipfile, "%15s", ipstr ) != EOF ) {
            if( allowed_chars_only( ipstr, sizeof( ipstr ), "0123456789." ) == FAILURE ) {
                continue;
            }
            
            //minimum 7 chars to form an ip string i.e. 1.2.3.4
            struct in_addr tmp;
            if( strlen( ipstr ) < 7 || inet_aton( ipstr, &tmp ) == 0 ) {
                continue;
            }
            
            _num_landmarks++;
            _landmarks = (struct in_addr*)realloc( _landmarks, _num_landmarks * sizeof( struct in_addr ) ); 
            assert( _landmarks != NULL );
            _landmarks[_num_landmarks - 1] = tmp;
        }
        
        fclose( ipfile );
        
        return SUCCESS;
    }
    else {
        error( "invalid input" );
        return FAILURE;
    }
}

struct in_addr landmarks_get( void )
{
    struct in_addr rlm;
    memset( &rlm, 0x00, sizeof( struct in_addr ) );

    if( _landmarks != NULL && _num_landmarks > 0 ) {
        rlm = _landmarks[get_random_uint( _num_landmarks )];
    }

    return rlm;
}

int landmarks_isloaded( void )
{
    return ( ( _landmarks != NULL && _num_landmarks > 0 ) ? TRUE:FALSE );
}

void landmarks_cleanup( void )
{
    if( _landmarks != NULL ) {
        free( _landmarks );
        _landmarks = NULL;
        _num_landmarks = 0;
    }
    else {
        _landmarks = NULL;
        _num_landmarks = 0;
    }
}

