/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */
#include <sys/wait.h>

#include "webcam.h"
#include "log.h"
#include "options.h"
#include "util.h"

#define WEBCAM_PHOTOFILE_PATH "/tmp/adeona.snapshot"
#define WEBCAM_PROGRAM_CMD " -w 160 -h 120 -t jpg" " " WEBCAM_PHOTOFILE_PATH " > /dev/null 2> /dev/null"

static int take_snapshot( void )
{
    char* cmd = concatstr( opts.webcamprog_path, WEBCAM_PROGRAM_CMD );
    int status = system( cmd );
    free( cmd );
    cmd = NULL;

    return ( WEXITSTATUS( status ) == EXIT_FAILURE ) ? FAILURE:SUCCESS;
}

int webcam_snapshot_get( webcamimg_t* img OUT )
{
    if( img != NULL && img->image != NULL ) {

        if( take_snapshot() == FAILURE ) {
            error( "could not take a snapshot" );
            return FAILURE;
        }

        FILE* imgfile = fopen( WEBCAM_PHOTOFILE_PATH, "r" );
        
        if( imgfile == NULL ) {
            error( "could not open image file" );
            system( "rm -f " WEBCAM_PHOTOFILE_PATH );
            img->len = 0;
            return FAILURE;
        }
        
        fseek( imgfile, 0, SEEK_END );
        long iflen = ftell( imgfile );
        if( iflen < 0 ) {
            error( "could not get img size" );
            system( "rm -f " WEBCAM_PHOTOFILE_PATH );
            img->len = 0;
            return FAILURE;
        }
        
        rewind( imgfile );
        
        int iread = -1;
        uchar* iptr = img->image;
        memset( img->image, 0x00, WEBCAM_IMGDATA_UPPERBOUND );
        while( ( iread = fread( iptr, sizeof( uchar ), iflen, imgfile ) ) > 0 ) {
            iptr += iread;
        }
        
        if( !feof( imgfile ) ) {
            error( "could not read entire image file" );
            system( "rm -f " WEBCAM_PHOTOFILE_PATH );
            memset( img->image, 0x00, WEBCAM_IMGDATA_UPPERBOUND );
            img->len = 0;
            return FAILURE;
        }
        
        fclose( imgfile );
        img->len = iflen;
        system( "rm -f " WEBCAM_PHOTOFILE_PATH );
        
        return SUCCESS;
    }
    else {
        error( "invalid input" );
        return FAILURE;
    }
}

int webcam_snapshot_save( const char* filename IN, webcamimg_t* img IN )
{
    if( filename != NULL && img != NULL && img->image != NULL && img->len > 0 ) {
        FILE* imgfile = fopen( filename, "w" );
        if( imgfile == NULL ) {
            error( "could not open file for saving" );
            return FAILURE;
        }

        int iwrite = 0;
        uchar* iptr = img->image;
        iwrite = fwrite( iptr, sizeof( uchar ), img->len, imgfile );
        
        if( iwrite < img->len ) {
            error( "could not write entire image file" );
            fclose( imgfile );
            return FAILURE;
        }
        
        fclose( imgfile );

        return SUCCESS;
    }
    else {
        error( "invalid input" );
        return FAILURE;
    }
}

