/*
 * Copyright (c) 2007-2008 University of Washington
 * 
 * All rights reserved. For information, contact the Adeona research
 * team at adeona@cs.washington.edu
 *
 * See the file License.txt included in this distribution for details.
 *
 */
#include <netdb.h>
#include <arpa/inet.h>

#include "common.h"
#include "opendht.h"
#include "log.h"
#include "util.h"
#include "options.h"

static void conn( struct in_addr serverip )
{
    printout( "connecting... " );
    if( opendht_connect( serverip, OPENDHT_PORT ) == DHT_FAILURE ) {
        fatal( "cannot connect" );
    }
    printout( "connected to server %s\n", inet_ntoa( serverip ) );

}

static void test_putget( void )
{
    gateways_load( "resources/gateways.adeona" );
    if( gateways_isloaded() == FALSE ) {
        printout( "could not load gateways file\n" );
        return;
    }
    
    struct hostent* he;
    he = gethostbyname( gateways_get_random() );
    if( he == NULL ) {
        fatal( "unknown host" );
    }
    
    dhtkey_t randomkey;
    get_random_bytes( randomkey, sizeof( dhtkey_t ) );
    
    int i;
    int numinsert = 10;
    uchar data[numinsert][OPENDHT_MAX_PUTLEN];
    for( i=0; i<numinsert; i++ ) {
        get_random_bytes( data[i], sizeof( data[i] ) );
//        log_msg_bin( DEBUG, data[i], sizeof( data[i] ), "put: " );
    }
    
    struct in_addr serverip = *(struct in_addr*)he->h_addr_list[0];
    
    printout( "doing puts...\n" );
    conn( serverip );
    int ret;
    i = 0;
    int ptry = 0;
    int MAX_TRY = 3 * numinsert;
    int retcode[numinsert];
    memset( retcode, 0x00, sizeof( int ) * numinsert );

    do {
        ret = opendht_put( randomkey, data[i], sizeof( data[i] ) );
        if( ret == DHT_SUCCESS ) { 
            retcode[i] = 1;
            i++; 
        }
        
        ptry++;
    } while( ( ret == DHT_TIMEOUT || i < numinsert ) && ptry < MAX_TRY );
    opendht_disconnect();
    
    int countsuccess = 0;
    for( i=0; i<numinsert; i++ ) {
        if( retcode[i] == 1 ) {
            countsuccess++;
        }
    }
    printout( "successful chunks put: %d out of %d\n", countsuccess, numinsert );

    printout( "doing get...\n" );
    uchar* result = NULL;
    int rlen = 0;
    do {
        conn( serverip );
        ret = opendht_get( randomkey, &result, &rlen );
        opendht_disconnect();
    } while( ( ret == DHT_TIMEOUT || ret == DHT_FAILURE ) && ptry < MAX_TRY  ) ;
    
    if( result != NULL ) { 
//        log_msg_bin( DEBUG, result, rlen, "get: " );
        if( rlen != ( numinsert * sizeof( data[0] ) ) ) {
            printout( "data get incomplete, %d out of %d bytes\n", rlen, numinsert * sizeof( data[0] ) );
            free( result );
            return;
        }

        countsuccess = 0;
        //opendht returns oldest first...
        uchar* cur = result;
        for( i=0; i<numinsert; i++ ) {
            if( memcmp( data[i], cur, sizeof( data[i] ) ) == 0 ) {
                countsuccess++;
            }
            cur += sizeof( data[i] );
        }   
        free( result );
        printout( "successful chunks get: %d out of %d\n", countsuccess, numinsert );
        if( countsuccess == numinsert ) {
            printout( "successfully put/get all chunks\n" );
        }
    }
    else {
        printout( "failed, no result\n" );
    }
    
}

static void load_gateways( void )
{
    gateways_load( "resources/gateways" );
    assert( gateways_isloaded() == TRUE );
    int i;
    for( i=0; i<10; i++ ) {
        printout( "%s\n", gateways_get_random() );
    }

    gateways_free();

    assert( gateways_isloaded() == FALSE );

}


int main( void )
{
    opts.have_console = TRUE;
    test_putget();
//    load_gateways();

    exit( EXIT_SUCCESS );
}
