#!/bin/bash
ADEONAPREFIX=/usr/local/adeona
RETRIEVESDIR=$HOME/Desktop/adeonaretrieval-`date "+%m-%d-%Y-%H%M"`

# uses System Events because of some bug with 'tell application "Terminal"' on 10.4
function get_file_path ()
{
    prompt="$1";
    path="$2";
    defaultname="$3"
    retval=`/usr/bin/osascript 2> /dev/null << EOT
        tell application "System Events"
            activate
            set ost to choose file with prompt "$prompt\n( if unsure, choose '$defaultname' )" default location POSIX file "$path"
            return POSIX path of ost
        end tell
    EOT`    
    if [ $? == 1 -o "$retval" == "" ]; then
        echo "FAILURE";
    else
        echo $retval;
    fi
}

function get_string ()
{
    msg="$1";
    title="$2";
    val=`/usr/bin/osascript 2> /dev/null << EOT
    tell application "System Events"
        activate
        set str to display dialog "$msg" default answer "" with title "$title"
        return str
    end tell
    EOT`
    retval=`expr "$val" : 'text returned:\(.*\),'`; 
    if [ $? == 1 -o "$retval" == "" ]; then
        echo "FAILURE";
    else
        echo $retval;
    fi  
}

# this needs to set focus back to Terminal because of the friggin 10.4 bug
function msg_box ()
{
    msg="$1";
    title="$2";
/usr/bin/osascript > /dev/null 2> /dev/null << EOT
    tell application "System Events"
        activate
        display dialog "$msg" buttons {"OK"} default button 1 with title "$title"
    end tell
    tell application "System Events"
        set frontmost of process "Terminal" to true
    end tell
EOT
}

#####################################################################

clear;
echo;
echo "Requesting retrieval parameters...";

# ask for parameters
OSTFILE=`get_file_path "Please select the location-finding credentials to be used" "/usr/local/adeona/adeona-retrievecredentials.ost" "adeona-retrievecredentials.ost"`
if [ "$OSTFILE" == "FAILURE" ]; then
    msg_box "Adeona encountered an error obtaining your location-finding credentials and cannot continue." "Location-finding Credentials Error"; 
    exit 1;
fi

#OPTSFILE=`get_file_path "Please select the Adeona options to be used" "/usr/local/adeona/resources/options.adeona" "options.adeona"`
#if [ "$OPTSFILE" == "FAILURE" ]; then
#    msg_box "Adeona encountered an error obtaining your Adeona options and cannot continue." "Adeona Options Error";
#    exit 1;
#fi

STARTTIME=`get_string "Please enter start time (MM/DD/YYYY,HH:MM). Leaving the box blank will default to 2 days ago" "Start time"`

ENDTIME=`get_string "Please enter end time (MM/DD/YYYY,HH:MM). Leaving the box blank will default to now" "End time"`

RECENTORALL=`/usr/bin/osascript << EOT
    tell application "System Events"
        activate
        set whichupdate to display dialog "You may specify the number of most recent locations to retrieve or, choose to retrieve all of the locations within the time period you specified." default answer "1" buttons {"All locations", "Recent locations"} default button 2 with title "Select Locations"
        if button returned of whichupdate is "All locations" then
            return "ALL"
        end if
        return text returned of whichupdate
    end tell
EOT`

#get password
#PWDENTRY=`/usr/bin/osascript 2> /dev/null << EOT
#    tell application "System Events"
#        activate
#        set wherepwd to display dialog "Adeona retrieval is ready to start. You may provide your Adeona password through a dialog box or more securely, directly to the Adeona retrieval program via the Adeona Terminal window." buttons {"Terminal", "Dialog"} default button 2 with title "Password Entry"
#        return button returned of wherepwd
#    end tell
#EOT`
#

#folder for the results
if [ ! -d $RETRIEVESDIR ]; then
    mkdir $RETRIEVESDIR;
fi;

#if [ "$PWDENTRY" == "Terminal" ]; then ##### terminal adeona-retrieve
#/usr/bin/osascript 2> /dev/null << EOT
#    tell application "System Events"
#        set frontmost of process "Terminal" to true
#    end tell
#EOT
#
#clear;
#echo; 
#echo "Please enter your Adeona password when prompted";
#if [ "$RECENTORALL" == "ALL" ]; then
#    $ADEONAPREFIX/adeona-retrieve.exe -r $ADEONAPREFIX/resources/ -l $RETRIEVESDIR/ -s $OSTFILE -t $STARTTIME -e $ENDTIME;
#else
#    $ADEONAPREFIX/adeona-retrieve.exe -r $ADEONAPREFIX/resources/ -l $RETRIEVESDIR/ -s $OSTFILE -t $STARTTIME -e $ENDTIME -n $RECENTORALL;
#fi
#
#else ######################################### dialog adeona-retrieve

PASSWD=`/usr/bin/osascript 2> /dev/null << EOT
    tell application "System Events"
        set pass to display dialog "Please enter your Adeona password." buttons{"OK"} default button 1 default answer "" with hidden answer with title "Adeona Password"
        return text returned of pass
    end tell
EOT`

/usr/bin/osascript 2> /dev/null << EOT
    tell application "System Events"
        set frontmost of process "Terminal" to true
    end tell
EOT

clear;
echo;
#if you are concerned about "-p" below, please consult our website at http://adeona.cs.washington.edu/faq.html
if [ "$RECENTORALL" == "ALL" ]; then
    if [ "$PASSWD" != "" ]; then
        $ADEONAPREFIX/adeona-retrieve.exe -r $ADEONAPREFIX/resources/ -l $RETRIEVESDIR/ -s $OSTFILE -t $STARTTIME -e $ENDTIME -p $PASSWD;
    else
        echo "No password given";
        echo;
        $ADEONAPREFIX/adeona-retrieve.exe -r $ADEONAPREFIX/resources/ -l $RETRIEVESDIR/ -s $OSTFILE -t $STARTTIME -e $ENDTIME;
    fi
else
    if [ "$PASSWD" != "" ]; then
        $ADEONAPREFIX/adeona-retrieve.exe -r $ADEONAPREFIX/resources/ -l $RETRIEVESDIR/ -s $OSTFILE -t $STARTTIME -e $ENDTIME -n $RECENTORALL -p $PASSWD;
    else
        echo "No password given";
        echo;
        $ADEONAPREFIX/adeona-retrieve.exe -r $ADEONAPREFIX/resources/ -l $RETRIEVESDIR/ -s $OSTFILE -t $STARTTIME -e $ENDTIME -n $RECENTORALL;
    fi
fi

#fi ###################################### end adeona-retrieve madness

# Done
if [ $? != 0 ]; then
    msg_box "Adeona encountered a problem during retrieval. Please check your password and retrieval parameters then try again." "Retrieval Error"
    rm -rf $RETRIEVESDIR;
else
    sleep 2;
    msg_box "Results have been stored in,\n'`basename $RETRIEVESDIR`'\n located on your Desktop. Please dispose of the information properly when done." "Retrieval complete";
    open -a Finder $RETRIEVESDIR;
fi

