#!/bin/bash
#
# Copyright (c) 2007-2008 University of Washington
# 
# All rights reserved. For information, contact the Adeona research
# team at adeona@cs.washington.edu
#
# See the file License.txt included in this distribution for details.
#

ADEONAPREFIX=/usr/local
INSTALLDIR=$ADEONAPREFIX/adeona

ADEONARESOURCES=$INSTALLDIR/resources
ADEONALOGS=$INSTALLDIR/logs
ADEONASCRIPTS=$INSTALLDIR/scripts

#CRONTABFILE=/etc/crontab
CRONTABENTRY="@reboot $INSTALLDIR/adeona-client.exe -s $INSTALLDIR/adeona-clientstate.cst -r $ADEONARESOURCES/ -l $ADEONALOGS/ &"


#####################################################################
if [ ! -e './configure' ]; then
    echo;echo "Please run $0 from the adeona project directory";echo;
    exit 1;
fi

if [ ! -e './config.h' ]; then
    echo;echo "Please run ./configure to generate config.h";
    echo "Please make sure there are no errors from the configure script.";echo;
    exit 1;
fi

clear;

#
# existing?
#
if [ -d $INSTALLDIR -a -e $INSTALLDIR/adeona-client.exe ]; then
    echo; echo "You have an existing installation of Adeona. Would you like to overwrite?";
    echo;
    while [ 1 ]; do
        echo -n "Enter 'y' to overwrite or 'n' to exit installation: "
        read response;
        if [ "$response" == "y" ]; then
            break;
        elif [ "$response" == "n" ]; then
            echo; echo "Exiting installation...";
            exit 0;
        else
            echo "Sorry, invalid response"; echo;
        fi
    done
    
    oldclient=`ps axo pid,command | grep adeona-client.exe | grep -v grep | { read pid dontcare; echo $pid; }`;
    if [ "$oldclient" != "" ]; then
        kill -9 $oldclient 2>&1 > /dev/null;
    fi

    rm -rf $INSTALLDIR;

#    OLDCRONENTRY=`grep -n adeona-client.exe $CRONTABFILE | cut -f1 -d:`;
#    sed -i -e "$OLDCRONENTRY,$OLDCRONENTRY d" $CRONTABFILE;
    echo;echo "Your old Adeona installation has been removed";
    echo "Please make sure to update your crontab entry for adeona-client.exe at the end of the installation.";
    echo;echo -n "Press any key to continue..."; read;
fi

#
# license agreement
#
clear;
echo;
cat License.txt;
echo;
while [ 1 ]; do
    echo -n "Enter 'y' to agree or 'n' to exit the installation: "
    read response;
    if [ "$response" == "y" ]; then
        echo; echo "Starting Adeona installation...";
        break;
    elif [ "$response" == "n" ]; then
        echo; echo "Exiting installation...";
        exit 0;
    else
        echo "Sorry, invalid response"; echo;
    fi
done

#
# build 
# 
if [ ! -e adeona-init.exe -o ! -e adeona-client.exe -o ! -e adeona-retrieve.exe ]; then
    make RELEASE=1;
fi

if [ ! -e adeona-init.exe -o ! -e adeona-client.exe -o ! -e adeona-retrieve.exe ]; then
    echo "Could not build Adeona programs.";
    exit 1;
fi

#
# make directories
#
if [ ! -d $INSTALLDIR ]; then
    mkdir $INSTALLDIR;
    mkdir $ADEONARESOURCES;
    mkdir $ADEONALOGS;
    mkdir $ADEONASCRIPTS;
fi

if [ ! -d $INSTALLDIR ]; then
    echo "Could not create installation directory. Please check your permissions.";
    echo "Try 'sudo make install'";
    exit 1;
fi

#
# copy files
#
mv adeona-init.exe $INSTALLDIR;
mv adeona-client.exe $INSTALLDIR;
mv adeona-retrieve.exe $INSTALLDIR;

cp resources/gateways.adeona $ADEONARESOURCES;
cp resources/landmarks.adeona $ADEONARESOURCES;
cp resources/options.adeona $ADEONARESOURCES;

cp Readme.txt $INSTALLDIR;

#
# initialize Adeona
#
SAVEWD=$PWD;
cd $INSTALLDIR;

clear;
echo; echo "To protect your location-finding credentials, please pick a password for Adeona. It does not need to be the same as your login password.";
echo;
./adeona-init.exe -r $ADEONARESOURCES/ -l $ADEONALOGS/;

if [ $? == 0 ]; then
    echo; echo "Adeona has been successfully initialized";
    cd $SAVEWD;
else
    cd $SAVEWD;
    rm -rf $INSTALLDIR;
    echo; echo "Adeona could not be initialized. Please make sure you gave a password.";
    echo;
    exit 1;
fi;

#
# start the client
#
echo;echo "Starting Adeona client..."
$INSTALLDIR/adeona-client.exe -s $INSTALLDIR/adeona-clientstate.cst -r $ADEONARESOURCES/ -l $ADEONALOGS/ &

#
# Remind about cron
#
clear;
echo;echo "The Adeona client program relies on cron to make sure it runs during system startup."
echo;echo "IMPORTANT: Please make sure to add the following entry to your crontab file via 'sudo crontab -e'";
echo;echo "$CRONTABENTRY";
echo;echo -n "Press any key to continue..."; read;
echo;

#
# done
#
echo;
if [ -d ~/Desktop ]; then
    cp $INSTALLDIR/adeona-retrievecredentials.ost ~/Desktop;
    cat Conclusion.txt;
else
    cp $INSTALLDIR/adeona-retrievecredentials.ost ~;
    sed -e 's/on your Desktop/in your home directory/' Conclusion.txt;
fi

